/* PCI-LANNER Libary Function.

   Author: Golden Wang

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   Use vi or gvim to edit this file and set tabstop=4.

   v 0.0.0  4 Jun 2010 by Golde Wang
     function created for PCI-LANNER DI/O, AI .

*/

#include "pcidio.h"

/* extern from pcidio.c */
extern char device_module[LINE_SIZE + 1];


/* declare for pcilanner.c */
#define CSBIT  1
#define SKBIT  2
#define DIBIT  4
#define DOBIT  0x04
#define RangeBase 56

static int Lanner_Gain[4];
static int Lanner_Offset[4];
static unsigned int timeout_max = 100, timeout_cal;

int PCIDA_EEP;
void SetCS(int fd);
void ClrCS(int fd);
void SetSK(int fd);
void ClrSK(int fd);
void SetDI(int fd);
void ClrDI(int fd);
int GetDO(int fd);
void CommuEEP(int fd);
DWORD GetTimeTicks_ms(void);

int Get_Lanner_Gain_Offset(WORD fd)
{
        int i;
        WORD read_value;
	int rs = 0;

        for(i = 0; i < 4; i++)
        {
                PCI_LANNER_EEPROM_ReadWord(fd, i, &read_value);

		if(read_value > 30000) 
		{
			rs |= (1<< i);
			read_value = 8192;
		}

                Lanner_Gain[i] = read_value;

                PCI_LANNER_EEPROM_ReadWord(fd, (i+4), &read_value);

		if((read_value > 60) || (read_value < -60)) 
		{
			rs |= (1<<(i+4));
			read_value = 0;
		}

                Lanner_Offset[i] = read_value;
        }

	return rs;
/*
        for(i=0 ; i < 8; i++)
        {
                PCI_LANNER_EEPROM_WriteEnable(fd);
                PCI_LANNER_EEPROM_WriteWord(fd, i+RangeBase, 0);
                PCI_LANNER_EEPROM_WriteDisable(fd);
        }
*/
}

WORD PCI_LANNER_Digital_Output(WORD fd, BYTE data)
{
	if (PCIDA_OutputByte(fd, IXPCI_DO, data)) 
		return PCIDA_LANNER_DIGITAL_OUTPUT_ERROR;

	usleep(1000);

	return PCIDA_NOERROR;
}

WORD PCI_LANNER_Digital_Input(WORD fd, BYTE *di_data)
{
	DWORD reg_value;

	reg_value = PCIDA_InputByte(fd, IXPCI_DI);

	if ( reg_value == FAILURE ) 
		return PCIDA_LANNER_DIGITAL_INPUT_ERROR;

	*di_data = reg_value;

	return PCIDA_NOERROR;
}

WORD PCI_LANNER_Read_Count(WORD fd, WORD count, DWORD *count_value)
{
	DWORD reg_value;

	if (count == 0)
        {
		reg_value = PCIDA_InputByte(fd, IXPCI_READ_C0);
        }
        else if (count == 1)
        {
		reg_value = PCIDA_InputByte(fd, IXPCI_READ_C1);
        }
        else
        {
                return PCIDA_LANNER_COUNTER_NUMBER_ERROR;
        }

	if ( reg_value == FAILURE )
        	return PCIDA_LANNER_READ_COUNTER_ERROR;
	

	*count_value = reg_value;

        return PCIDA_NOERROR;
}

WORD PCI_LANNER_Clear_Count(WORD fd, WORD count)
{
	if (count == 0)
	{  
		if (PCIDA_OutputByte(fd, IXPCI_CLEAR_C0, 0xff)) 
			return PCIDA_LANNER_CLEAR_COUNTER_ERROR;
	}
	else if (count == 1)
	{
		if (PCIDA_OutputByte(fd, IXPCI_CLEAR_C1, 0xff)) 
			return PCIDA_LANNER_CLEAR_COUNTER_ERROR;
	}
	else
	{
		return PCIDA_LANNER_COUNTER_NUMBER_ERROR;
	}

	return PCIDA_NOERROR;
}

WORD PCI_LANNER_Set_Voltage_Gain_MUX(WORD fd, WORD channel, WORD gain)
{
	WORD reg_value;

	if ( channel > 3 || channel < 0 )
		return PCIDA_LANNER_ANALOG_CHANNEL_ERROR;
	
	if ( gain > 3 || gain < 0 )
		return PCIDA_LANNER_ANALOG_GAIN_ERROR;
	                        
	//Setting Analog Voltage Channel and Gain
	reg_value = (gain << 2) + channel;
	
	if (PCIDA_OutputByte(fd, IXPCI_ADGCR, reg_value))
		return PCIDA_LANNER_SET_CHANNEL_GAIN_ERROR;
        
	return PCIDA_NOERROR;
}

WORD PCI_LANNER_Set_Current_Gain_MUX(WORD fd, WORD channel)
{
	WORD gain = 2;
	WORD reg_value;

        if ( channel > 3 || channel < 0 )
                return PCIDA_LANNER_ANALOG_CHANNEL_ERROR;
        
        //Setting Analog Current Channel and Gain
        reg_value = (gain << 2) + channel;
        
        if (PCIDA_OutputByte(fd, IXPCI_ADGCR, reg_value))
        	return PCIDA_LANNER_SET_CHANNEL_GAIN_ERROR;
        
        return PCIDA_NOERROR;
}

WORD PCI_LANNER_ReadAI_Hex(WORD fd, WORD *hex_value)
{
	WORD bHi, bLo;
        WORD ai_hex;
        WORD reg_value;

        //Setting Polling Mode 
        if (PCIDA_OutputByte(fd, IXPCI_ADPR, 0xff))
        	return PCIDA_LANNER_SET_POLLING_ERROR;
        
	timeout_cal = 0;

	//usleep(5);
	while(1)
	{
        	if(PCIDA_InputByte(fd, IXPCI_ADGCR))	//AD Ready
		{
			if(timeout_cal > timeout_max)
				timeout_max = timeout_cal;
			break;
		}
		else
		{
			timeout_cal++;

			if(timeout_cal > (timeout_max * 10))
			{
				return PCIDA_LANNER_ANALOG_INPUT_ERROR;
			}
			
		}
	}	
        
        //Read AI HEX Value
        ai_hex = PCIDA_InputByte(fd, IXPCI_AD);

	*hex_value = ai_hex;
        
	return PCIDA_NOERROR;
}

WORD PCI_LANNER_Read_Voltage(WORD fd, WORD channel, WORD gain, float *fvalue)
{       
        WORD bHi, bLo;
        short ai_hex;
	WORD reg_value;

	if ( channel > 3 || channel < 0 )
                return PCIDA_LANNER_ANALOG_CHANNEL_ERROR;

        if ( gain > 3 || gain < 0 )
                return PCIDA_LANNER_ANALOG_GAIN_ERROR;

        //Setting Polling Mode 
        if (PCIDA_OutputByte(fd, IXPCI_ADPR, 0xff))
        	return PCIDA_LANNER_SET_POLLING_ERROR;

	timeout_cal = 0;
	
	while(1)
        {
                if(PCIDA_InputByte(fd, IXPCI_ADGCR))    //AD Ready
                {
                        if(timeout_cal > timeout_max)
                                timeout_max = timeout_cal;
                        break;
                }
                else
                {
                        timeout_cal++;

                        if(timeout_cal > (timeout_max * 10))
                        {
                                return PCIDA_LANNER_ANALOG_INPUT_ERROR;
                        }

                }
        }

        //Read AI HEX Value
        ai_hex = PCIDA_InputByte(fd, IXPCI_AD);
        ai_hex = ai_hex << 4;
	ai_hex = ai_hex >> 4;

	switch(gain)
	{
		case 0:
			*fvalue = (ai_hex * 10) / (float)2048;	 //AI*10/0x7ff
			break;
		case 1:
			*fvalue = (ai_hex * 5) / (float)2048;	 //AI*5/0x7ff
			break;
		case 2:
			*fvalue = (ai_hex * 2.5) / (float)2048;	 //AI*2.5/0x7ff
			break;
		case 3:
			*fvalue = (ai_hex * 1.25) / (float)2048; //AI*1.25/0x7ff
			break;
        }

        return PCIDA_NOERROR;
}       

WORD PCI_LANNER_Read_CalVoltage(WORD fd, WORD channel, WORD gain, float *fvalue)
{
	WORD tmp_ai_hex;
        long cal_ai_hex;
	short uncal_ai_hex;
	short act_gain, act_offset;

	if ( channel > 3 || channel < 0 )
                return PCIDA_LANNER_ANALOG_CHANNEL_ERROR;

        if ( gain > 3 || gain < 0 )
                return PCIDA_LANNER_ANALOG_GAIN_ERROR;

        //Setting Polling Mode 
	if (PCIDA_OutputByte(fd, IXPCI_ADPR, 0xff))
        	return PCIDA_LANNER_SET_POLLING_ERROR;

	timeout_cal = 0;

	while(1)
        {
                if(PCIDA_InputByte(fd, IXPCI_ADGCR))    //AD Ready
                {
                        if(timeout_cal > timeout_max)
                                timeout_max = timeout_cal;
                        break;
                }
                else
                {
                        timeout_cal++;

                        if(timeout_cal > (timeout_max * 10))
                        {
                                return PCIDA_LANNER_ANALOG_INPUT_ERROR;
                        }

                }
        }

	//Get EEPROM Action Gain and Offset
	act_gain = Lanner_Gain[gain];
        act_offset = Lanner_Offset[gain];

        //Read AI HEX Value
	PCI_LANNER_ReadAI_Hex(fd, &tmp_ai_hex);
	uncal_ai_hex = tmp_ai_hex << 4;
	uncal_ai_hex = uncal_ai_hex >> 4;
	cal_ai_hex = uncal_ai_hex * act_gain;
	cal_ai_hex = cal_ai_hex >> 13;
	act_offset = act_offset >> 4;
	cal_ai_hex = cal_ai_hex + act_offset;

	if( cal_ai_hex & 0x800 )
	{
		if(cal_ai_hex < -2048)
			cal_ai_hex = -2048;

	}
	else
	{
		if(cal_ai_hex > 2047)
			cal_ai_hex = 2047;
	}

	switch(gain)
        {
                case 0:
                        *fvalue = (cal_ai_hex * 10) / (float)2048;   //AI*10/0x7ff
                        break;
                case 1:
                        *fvalue = (cal_ai_hex * 5) / (float)2048;    //AI*5/0x7ff
                        break;
                case 2:
                        *fvalue = (cal_ai_hex * 2.5) / (float)2048;  //AI*2.5/0x7ff
                        break;
                case 3:
                        *fvalue = (cal_ai_hex * 1.25) / (float)2048; //AI*1.25/0x7ff
                        break;
        }

        return PCIDA_NOERROR;
}


WORD PCI_LANNER_Read_Current(WORD fd, WORD channel, float *fvalue)
{       
        WORD bHi, bLo;
        WORD ai_hex;
	WORD gain = 2;
        WORD reg_value;

	if ( channel > 3 || channel < 0 )
                return PCIDA_LANNER_ANALOG_CHANNEL_ERROR;

        //Setting Polling Mode 
        if (PCIDA_OutputByte(fd, IXPCI_ADPR, 0xff))
        	return PCIDA_LANNER_SET_POLLING_ERROR;
        
	timeout_cal = 0;

	while(1)
        {
                if(PCIDA_InputByte(fd, IXPCI_ADGCR))    //AD Ready
                {
                        if(timeout_cal > timeout_max)
                                timeout_max = timeout_cal;
                        break;
                }
                else
                {
                        timeout_cal++;

                        if(timeout_cal > (timeout_max * 10))
                        {
                                return PCIDA_LANNER_ANALOG_INPUT_ERROR;
                        }

                }
        }

        //Read AI HEX Value
        ai_hex = PCIDA_InputByte(fd, IXPCI_AD);
        //printf("Current HexValue : 0x%03x\n",ai_hex);

	*fvalue = (ai_hex * 20) / (float)2048;
        
        return PCIDA_NOERROR;
}       

WORD PCI_LANNER_Read_CalCurrent(WORD fd, WORD channel, float *fvalue)
{
	WORD tmp_ai_hex;
        long cal_ai_hex;
        short uncal_ai_hex;
        short act_gain, act_offset;
        WORD gain = 2;

        if ( channel > 3 || channel < 0 )
                return PCIDA_LANNER_ANALOG_CHANNEL_ERROR;

	if (PCIDA_OutputByte(fd, IXPCI_ADPR, 0xff))
                return PCIDA_LANNER_SET_POLLING_ERROR;

	timeout_cal = 0;

	while(1)
        {
                if(PCIDA_InputByte(fd, IXPCI_ADGCR))    //AD Ready
                {
                        if(timeout_cal > timeout_max)
                                timeout_max = timeout_cal;
                        break;
                }
                else
                {
                        timeout_cal++;

                        if(timeout_cal > (timeout_max * 10))
                        {
                                return PCIDA_LANNER_ANALOG_INPUT_ERROR;
                        }

                }
        }

	//Get EEPROM Action Gain and Offset
	act_gain = Lanner_Gain[gain];
	act_offset = Lanner_Offset[gain];

        //Read AI HEX Value
	PCI_LANNER_ReadAI_Hex(fd, &tmp_ai_hex);
        uncal_ai_hex = tmp_ai_hex << 4;
        uncal_ai_hex = uncal_ai_hex >> 4;
        cal_ai_hex = uncal_ai_hex * act_gain;
        cal_ai_hex = cal_ai_hex >> 13;
        act_offset = act_offset >> 4;
        cal_ai_hex = cal_ai_hex + act_offset;

	*fvalue = (cal_ai_hex * 20) / (float)2048;               

        return PCIDA_NOERROR;
}

WORD PCI_LANNER_Read_Voltage_Polling(WORD fd, WORD channel, WORD gain, DWORD datacount, float *voltage)
{
	unsigned cnt;
        DWORD start;
        float vol;
	WORD ret;

	ret = PCI_LANNER_Set_Voltage_Gain_MUX(fd, channel, gain);

	if(ret)
		return ret;

        if(datacount>32786)
		datacount=32786;

        start = GetTimeTicks_ms();

        for(cnt=0; cnt<datacount; cnt++)
        {
/*
                if(GetTimeTicks_ms()-start > 1)
                {
			//printf("GetTimeTicks_ms : %d, start : %d\n", GetTimeTicks_ms(), start);
			start = GetTimeTicks_ms();
			
			ret = PCI_LANNER_Set_Voltage_Gain_MUX(fd, channel, gain);

			if(ret)
				return ret;
                }
*/
	
		ret = PCI_LANNER_Read_Voltage(fd, channel, gain, &vol);

		if(ret)
			return ret;

		voltage[cnt] = vol;
	}

        return PCIDA_NOERROR;
}

WORD PCI_LANNER_Read_Current_Polling(WORD fd, WORD channel, DWORD datacount, float *current)
{
        unsigned cnt;
        DWORD start;
        float cur;
        WORD ret;

        ret = PCI_LANNER_Set_Current_Gain_MUX(fd, channel);

        if(ret)
                return ret;

        if(datacount>32786)
                datacount=32786;

        start = GetTimeTicks_ms();

        for(cnt=0;cnt<datacount; cnt++)
        {
/*
                if(GetTimeTicks_ms()-start > 1)
                {
			start = GetTimeTicks_ms();

                        ret = PCI_LANNER_Set_Current_Gain_MUX(fd, channel);

                        if(ret)
                                return ret;
                }
*/
		ret = PCI_LANNER_Read_Current(fd, channel, &cur);

                if(ret)
                        return ret;

                current[cnt] = cur;
        }

        return PCIDA_NOERROR;
}



void SetCS(int fd)
{
        PCIDA_EEP |= CSBIT;
        PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
}

void ClrCS(int fd)
{
        PCIDA_EEP &= ~CSBIT;
        PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
}

void SetSK(int fd)
{
        PCIDA_EEP |= SKBIT;
        PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
}

void ClrSK(int fd)
{
        PCIDA_EEP &= ~SKBIT;
        PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
}

void SetDI(int fd)
{
        PCIDA_EEP |= DIBIT;
        PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
}

void ClrDI(int fd)
{
        PCIDA_EEP &= ~DIBIT;
        PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
}

int GetDO(int fd)
{
        int val;

        val = PCIDA_InputByte(fd, IXPCI_EEP) & DOBIT;

        return val;
}

void CommuEEP(int fd)
{
        PCIDA_EEP = 0;

        SetDI(fd);
        SetCS(fd);
        SetSK(fd);
        ClrSK(fd);
}

WORD PCI_LANNER_EEPROM_WriteEnable(WORD fd)
{
        int i, Command;

        CommuEEP(fd);

        Command=0x30;

        for(i=7; i>=0; i--)
        {
                ClrSK(fd);
                if(Command & (1<<i))
                {
                        SetDI(fd);
                }
                else
                {
                        ClrDI(fd);
                }

                SetSK(fd);
        }

        ClrSK(fd);
        ClrCS(fd);
        ClrDI(fd);

        return PCIDA_NOERROR;
}

WORD PCI_LANNER_EEPROM_WriteDisable(WORD fd)
{
        int i, Command;

        CommuEEP(fd);

        Command=0;

        for(i=7; i>=0; i--)
        {
                ClrSK(fd);

                if(Command & (1<<i))
                {
                        SetDI(fd);
                }
                else
                {
                        ClrDI(fd);
                }

                SetSK(fd);
        }

        ClrSK(fd);
        ClrCS(fd);
        ClrDI(fd);

        return PCIDA_NOERROR;
}

WORD PCI_LANNER_EEPROM_WriteWord(WORD fd, WORD addr, WORD value)
{
        int i, j;
        ixpci_reg_t reg;

        if(addr > 63 || addr < 0)
                return PCIDA_LANNER_EEPROM_ADDR_ERROR;

        CommuEEP(fd);

        PCIDA_EEP = 0xff;
        addr = addr | 0x40;

        for(i = 7; i >= 0; i--)
        {
                ClrSK(fd);

                if(addr & (1<<i))
                {
                        SetDI(fd);
                }
                else
                {
                        ClrDI(fd);
                }

                SetSK(fd);
        }

	for(i = 15; i >= 0; i--)
        {
                ClrSK(fd);

                if(value & (1<<i))
                {
                        SetDI(fd);
                }
                else
                {
                        ClrDI(fd);
                }

                SetSK(fd);
        }

        ClrSK(fd);
        ClrCS(fd);
        ClrDI(fd);


        for( i = 0; i < 10; i++)
        {
                PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
        }

        SetCS(fd);

	for( i = 0; i < 10; i++)
        {
                PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);
        }

        for( i = 0; i < 100; i++)
                for( j = 0; j < 32767; j++)
                {
                        SetSK(fd);
                        if(GetDO(fd))
                        {
                                ClrCS(fd);
                                return PCIDA_NOERROR;
                        }

                        ClrSK(fd);
                }

        PCIDA_EEP = 0xf;

	PCIDA_OutputByte(fd, IXPCI_EEP, PCIDA_EEP);

        return PCIDA_LANNER_EEPROM_WRITE_ERROR;
}

WORD PCI_LANNER_EEPROM_ReadWord(WORD fd, WORD addr, WORD *value)
{
	int i;

	if(addr > 63 || addr < 0)
        	return PCIDA_LANNER_EEPROM_ADDR_ERROR;

        CommuEEP(fd);

        PCIDA_EEP = 0xf;

        addr = addr | 0x80;

        for(i = 7; i >= 0; i--)
        {
                ClrSK(fd);

                if(addr & (1<<i))
                {
                        SetDI(fd);
                }
                else
                {
                        ClrDI(fd);
                }

                SetSK(fd);
        }

	ClrDI(fd);
        ClrSK(fd);
        SetSK(fd);
        ClrSK(fd);

        *value = 0;

        PCIDA_EEP = 0xf;

        for(i = 0; i < 16; i++)
        {
                ClrSK(fd);
                (*value) <<= 1;

                if(GetDO(fd))
                {
			(*value) += 1;
                }
                else
                {

   		}
                SetSK(fd);
        }

        ClrSK(fd);
        ClrCS(fd);

        return PCIDA_NOERROR;
}

DWORD GetTimeTicks_ms(void)
{
        struct timeval time_ticks;
        gettimeofday(&time_ticks, NULL);
        return ((time_ticks.tv_sec * 1000) + (time_ticks.tv_usec/1000));
}

